(function () {
    if (window.DisneyJSBridge) {
        return;
    }

    var messagingIframe;
    var sendMessageQueue = [];
    var receiveMessageQueue = [];
    var messageHandlers = {};

    var CUSTOM_PROTOCOL_SCHEME = 'disneybridge';
    var QUEUE_HAS_MESSAGE = '__QUEUE_MESSAGE__/';

    var responseCallbacks = {};
    var uniqueId = 1;

    function _initIframe(doc) {
        messagingIframe = doc.createElement('iframe');
        messagingIframe.style.display = 'none';
        doc.documentElement.appendChild(messagingIframe);
    }

    //set default messageHandler
    function init(messageHandler) {
        console.log('DisneyJSBridgeReady init---->');
        if (DisneyJSBridge._messageHandler) {
            throw new Error('DisneyJSBridge.init called twice');
        }
        DisneyJSBridge._messageHandler = messageHandler;
        var receivedMessages = receiveMessageQueue;
        receiveMessageQueue = null;
        for (var i = 0; i < receivedMessages.length; i++) {
            _dispatchMessageFromNative(receivedMessages[i]);
        }
    }

    function call(handlerName, data, responseCallback) {
        if (!isString(handlerName))
            return;
        if (isJson(data) && isFunction(responseCallback)) {
            console.log('call(handlerName, data, responseCallback)');
            _doSend(constructMessage(handlerName, data), responseCallback);
        } else if (isFunction(data)) {
            console.log('call(handlerName, responseCallback)');
            _doSend(constructMessage(handlerName, {}), data);
        } else if (isJson(data)) {
            console.log('call(handlerName, data)');
            _doSend(constructMessage(handlerName, data), function () {});
        } else {
          console.log('can not support call type');
        }
    }

    function isString(str) {
        return str && str != undefined && str != null && typeof str == 'string';
    }

    function isJson(json) {
        return json && json != undefined && json != null && typeof json == 'object' && typeof json != 'function';
    }

    function isFunction(fun) {
        return fun && fun != undefined && fun != null && typeof fun == 'function';
    }

    function registerHandler(handlerName, handler) {
        messageHandlers[handlerName] = handler;
    }

    function constructMessage(handlerName, params) {
    		var JSONMessage = {
    			handlerName: handlerName,
    			data: params
    		};
    		return JSONMessage;
    }

    //sendMessage add message, trigger native to handle sendMessage
    function _doSend(message, responseCallback) {
        if (responseCallback) {
            var callbackId = 'cb_' + (uniqueId++) + '_' + new Date().getTime();
            responseCallbacks[callbackId] = responseCallback;
            message.callbackId = callbackId;
        }

        sendMessageQueue.push(message);
        messagingIframe.src = CUSTOM_PROTOCOL_SCHEME + '://' + QUEUE_HAS_MESSAGE;
    }

    // provide to native
    function _fetchQueue() {
        var messageQueueString = JSON.stringify(sendMessageQueue);
        sendMessageQueue = [];
        //android can't read directly the return data, so we can reload iframe src to communicate with java
        messagingIframe.src = CUSTOM_PROTOCOL_SCHEME + '://return/_fetchQueue/' + encodeURIComponent(messageQueueString);
    }

    function _dispatchMessageFromNative(messageJSON) {
        setTimeout(function () {
            var message = JSON.parse(messageJSON);
            var responseCallback;
            //java call finished, now need to call js callback function
            if (message.responseId) {
                responseCallback = responseCallbacks[message.responseId];
                if (!responseCallback) {
                    return;
                }
                responseCallback(JSON.parse(message.responseData));
                delete responseCallbacks[message.responseId];
            } else {
                if (message.callbackId) {
                    var callbackResponseId = message.callbackId;
                    responseCallback = function (responseData) {
                        _doSend({
                            responseId: callbackResponseId,
                            responseData: responseData
                        });
                    };
                }

                var handler = DisneyJSBridge._messageHandler;
                if (message.handlerName) {
                    handler = messageHandlers[message.handlerName];
                }
                try {
                    handler(message.data, responseCallback);
                } catch (exception) {
                    if (typeof console != 'undefined') {
                        console.log("DisneyJSBridge: WARNING: javascript handler threw.", message, exception);
                    }
                }
            }
        });
    }

    function _handleMessageFromNative(messageJSON) {
        console.log(messageJSON);
        if (receiveMessageQueue && receiveMessageQueue.length > 0) {
            receiveMessageQueue.push(messageJSON);
        } else {
            _dispatchMessageFromNative(messageJSON);
        }
    }

    var DisneyJSBridge = window.DisneyJSBridge = {
        init: init,
        registerHandler: registerHandler,
        call: call,
        _fetchQueue: _fetchQueue,
        _handleMessageFromNative: _handleMessageFromNative
    };

    var doc = document;
    _initIframe(doc);
    var readyEvent = doc.createEvent('Events');
    readyEvent.initEvent('DisneyJSBridgeReady');
    readyEvent.bridge = DisneyJSBridge;
    doc.dispatchEvent(readyEvent);
    //This is just to initialize the default handler. Almost no usage currently. Can research later.
    DisneyJSBridge.init(null);
})();
